# Generated by Django 3.0.3 on 2020-03-12 14:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0007_visitorkey'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='usergroup',
            name='authenticate_with_email_password',
        ),
        migrations.RemoveField(
            model_name='usergroup',
            name='authenticate_with_phone_otp',
        ),
        migrations.RemoveField(
            model_name='usergroup',
            name='authenticate_with_phone_password',
        ),
    ]
